##
## The following script reproduces the example in the
## vignette "auckland" from the package \pkg{spdep}
## using \pkg{aRT} to transfer the data and some
## results to a TL DB
##
## Additionally this text ilustrates operation on polygon
## geometry such as extracting metrics and neighbourhoods
##
## Last change: 20060610 Pedro

##
## 1. Getting started
##
require(sp)
require(aRT)

require(spdep)
data(auckland)

## results from a data-analysis using an \pkg{spdep} function
res <- probmap(auckland$Deaths.1977.85, 9*auckland$Under.5.1981)

#brks = c(-Inf, 2, 2.5, 3, 3.5, Inf)
#cols = 1:5
#rawx=cols[findInterval(res$raw*1000, brks)]

##
## 2. Transfering to the DBMS
##

## 2.1 Preparing data: converting to \pkg{sp}
res = cbind(auckland, res)
res = cbind(data.frame(id = paste(1:length(auckpolys)), res))


## IST THERE A BETTER WAY TO CONVERTO TO sp HERE?
# PEDRO: fiz um sapply. achoo q nao tem como ser melhor que isso agora,
# porque tenho que gerar o ID, e este nao tem como encontrar em auckpolys

polygons = sapply(1:length(auckpolys), function(x)
{
  Polygons(list(Polygon(auckpolys[[x]])),ID=paste(x))
})
SP <- SpatialPolygons(polygons)

#polygons=list()
#
#for(i in 1:length(auckpolys))
#{
#  p = Polygon(auckpolys[[i]])
#  P = Polygons(list(p), ID = paste(i))
#  polygons[[i]]=P
#}
#
#SP <- SpatialPolygons(polygons)


## 2.2 Transfering the the TL BD

con = openConn()

if(any(showDbs(con) == "auckland"))
  deleteDb(con, "auckland", force=T)

#rm(db); gc()
db = createDb(con, "auckland")
db

l = createLayer(db, "districts")
addPolygons(l, SP)
t = createTable(l, "t_auck", gen=T)

updateColumns(t, res)

db

## plotting from the data-base
plot(l)
## CAN I SEE ATTRIBUTES
getData(t)[1:3,]

## WHAT THE DIFFERENCE BETWEEN plot() and aRTplot()
# PEDRO: com o aRTplot voce pode selecionar as geometrias baseada no ID,
# o que nao pode ser feito no sp porque nem todos os objetos tem ID
# (linhas e poligons tem, mas os pontos nao tem, tanto eh q no aRT nos usamos
# SpatialPointsDataFrame para jogar os dados no banco


## and the data can be seen in TV

##
## 3. Preparing visualisations for TerraView
##


## a theme with the original counts
th=createTheme(l, "rawData")

## and now reloading the data in TV we can see a visualisation
## of the data already there

## WHAT AND HOW DO I QUERY FOR ATTRIBUTE NAMES?
## CAN I QUERY BOTH TABLES AND THEMES? -- HOW?
# PEDRO:
thexpcount = createTheme(l,"texpcount",att = "expCount > 300")
plot(th)
plot(thexpcount,bor="red",add=T)


## and now "slicing the data" and adding a colorkey
## to the visualisation
v1=visualPolygons(color= c("#F7F7F7", "#CCCCCC", "#969696", "#636363", "#252525"), transp=0)
setVisual(th, v1, att="raw")

th=createTheme(l, "relativeRisk")
setVisual(th, v1, att="relRisk")

th=createTheme(l, "probabilityMap")
setVisual(th, v1, att="pmap")

th=createTheme(l, "expectedCounts")
setVisual(th, v1, att="expCount")

## WHAT THE SLICING CRITERIA?
## WHERE IS THIS DOCUMENTED?
# ?setVisual
# o criterio default eh intervalos igualmente espacados baseado no numero
# de cores utilizado como argumento

## CAN I CHANGE A PRE-EXISTING VISUAL ON THE SAME THEME?
## == IF NOT -- ISSUE AN ERROR MESSAGE WHEN TRYING!!!
# PEDRO: nao. tenho q colocar erro nisso, e talvez criar uma
# funcao do tipo removeVisual(theme)

## WHY DO I NEED trans=T (OTHERWISE I CANNOT SEE COLORS IN TV ?
# PEDRO: os poligonos no R sao transparentes por natureza.
# entao eu tinha colocado da mesma forma para o usuario construir
# no TerraView. Eles nao aparecem no terraView a menos que voce 
# use tranparencia menor que o valor default da funcao, que eh 100%.
# talvez seja melhor mudar o default para 100...

th=createTheme(l, "expectedCounts4")
setVisual(th, visualPolygons(color=terrain.colors(10), transp=0), att="expCount")

## CAN I CHANGE A PRE-EXISTING VISUAL ON THE SAME THEME?
## == APPARENTLY YES == ANY RESTRICTIONS OR COMMENTS


## some options when "slicing" data for visualisation
## INCLUDE OPTIONS FOR SLICING HERE!
# PEDRO: use a opcao 'mode' (?setVisual para outras)
th=createTheme(l, "QuantilexpectedCounts")
setVisual(th, v1, att="expCount", mode="quantil")



##
## 4. Some operations on polygons
##

## 4.1 computing polygons attributes

## area and perimeter of each district
area = as.data.frame(getMetric(l, "area"))
area

perimeter = as.data.frame(getMetric(l, "length"))
perimeter

## adding area and perimeter to the polygon attribute table 
createColumn(t, "area")
updateColumns(t, area)
createColumn(t, "length")
updateColumns(t, perimeter)

head(getData(t))

## IN WHAT IS ABOVE DOES THE NAMES OF TL TABLE AND IN THE OBJECT NEED
## TO BE THE SAME?
# ele casa os colnames do data.frame com as colunas da tabela

## Operations on polygon geometry

## centroids
center = getOperation(l, "centroid")
center
## storing centroids in another layer
lcentroids = importSpData(db,center, "centroids")

## neighborhood, choosing the target
target = c("2","18","73")
colors = terrain.colors(5)
pols = getPolygons(l)
aRTplot(pols)
aRTplot(pols,id=target,col=colors[1],add=T)

## Neighbours by touching,  1st and 2nd order
neigh = getRelation(l,"touches",id=target)
neigh2 = getRelation(l,"touches",id=neigh)

# nao funciona direito porque o dado nao esta certinho
# acho q na terralib tem uma variavel define um criterio
# de erro, entao tem que aumentar para passar aqui
# outra solucao eh usar o buffer, como nos vamos usar mais adiante
# o erro aqui pode ser tambem por causa da projecao (que nos nao estamos
# usando)
aRTplot(pols,id=target)
aRTplot(pols,add=T)
aRTplot(pols,id=neigh,col=colors[2],add=T)
aRTplot(pols,id=neigh2,col=colors[3],add=T)
aRTplot(pols,id=target,col=colors[1],add=T)

## Neighbours inside and crossing buffers
# (the buffer operation in the newest TerraLib is faster than this one)
# let us use a minor error (0.1) to define the neighbourhood criterion
buffer = getOperation(l,"buffer", dist=.1)
lb = importSpData(db,buffer,lname="buffer")

ov = getRelation(l,"overlaps",lb,id=target)
ov2 = getRelation(l,"overlaps",lb,id=ov)

aRTplot(pols)
aRTplot(pols,id=ov2,col=colors[3],add=T)
aRTplot(pols,id=ov,col=colors[2],add=T)
aRTplot(pols,id=target,col=colors[1],add=T)

## Neighbours within a buffer radius (bigger than the error)
buffer2 = getOperation(l,"buffer", id=target,dist=4)
lb2 = importSpData(db,buffer2,lname="buffer2")
aRTplot(pols,id=target,add=T)
ov = getRelation(l,"overlaps",lb2,id=target)
wit = getRelation(l,"within",lb2,id=target)

aRTplot(pols,col="gray",lty=0)
aRTplot(pols,id=wit,col=colors[3],add=T)
aRTplot(pols,id=ov,col=colors[2],add=T)
aRTplot(pols,id=target,col=colors[1],add=T)
aRTplot(buffer2,add=T,bor="red",lty=2)

##
## Tyding up everything
##
## DB stuff (DO WE NEED TO DELETE ALL SINCE ONE DEPENDS ON THE OTHER?)
# PEDRO: no, only 'db'. The other variables become invalid then.
##
## 5. Retrieving (importing) data from the DB 
##

##
## Assume now that the TL DB already exists and we want
## to import the data to R.

## lets start from the scratch from an empty workspace

rm(list=ls())
gc()


## connect to the DBMS
con = openConn()

## checking for existing DB's
showDbs(con)

## connecting to a specific DB
db = openDb(con, "auckland")

## inspecting the data base contents
## inspecting geometry (ies)
db

## and their associates attribute tables
l = openLayer(db,"districts")
t = openTable(l, "t_auck")

## importing data
polygons = getPolygons(l)
plot(polygons)
head(getData(t))




